from fastmcp import FastMCP
from pydantic import Field
from typing import Any
from okx.Trade import TradeAPI

from .config import *

ACCOUNT = TradeAPI(
    api_key=OKX_API_KEY,
    api_secret_key=OKX_API_SECRET,
    passphrase=OKX_PASSPHRASE,
    flag=OKX_TRADE_FLAG,
    domain=OKX_BASE_URL,
)


def add_tools(mcp: FastMCP):

    @mcp.tool(
        title="Place a new order",
        description="Place a new order on OKX for trading",
    )
    def place_order(
        instId: str = Field(description="Instrument ID, e.g. BTC-USDT"),
        tdMode: str = Field(description="Trade mode."
                                        "Margin mode: `cross`/`isolated`. Non-Margin mode: `cash`. "
                                        "\n`spot_isolated`: (only applicable to SPOT lead trading, `tdMode` should be `spot_isolated` for SPOT lead trading.)"
                                        "\nNote: `isolated` is not available in multi-currency margin mode and portfolio margin mode."),
        side: str = Field(description="Order side, `buy`/`sell`"),
        ordType: str = Field(description="Order type. "
                                         "\n`market`: Market order, only applicable to SPOT/MARGIN/FUTURES/SWAP"
                                         "\n`limit`: Limit order"
                                         "\n`post_only`: Post-only order"
                                         "\n`fok`: Fill-or-kill order"
                                         "\n`ioc`: Immediate-or-cancel order"
                                         "\n`optimal_limit_ioc`: Market order with immediate-or-cancel order (applicable only to Expiry Futures and Perpetual Futures)."
                                         "\n`mmp`: Market Maker Protection (only applicable to Option in Portfolio Margin mode)"
                                         "\n`mmp_and_post_only`: Market Maker Protection and Post-only order(only applicable to Option in Portfolio Margin mode)"),
        sz: str = Field(description="Quantity to buy or sell"),
        ccy: str = Field("", description="Margin currency. Applicable to all `isolated` `MARGIN` orders and `cross` `MARGIN` orders in `Futures mode`"),
        clOrdId: str = Field("", description="Client Order ID as assigned by the client."
                                             "A combination of case-sensitive alphanumerics, all numbers, or all letters of up to 32 characters."
                                             "Only applicable to general order. It will not be posted to algoId when placing TP/SL order after the general order is filled completely."),
        tag: str = Field("", description="Order tag. A combination of case-sensitive alphanumerics, all numbers, or all letters of up to 16 characters"),
        posSide: str = Field("", description="Position side. The default is `net` in the net mode. "
                                             "It is required in the `long/short` mode, and can only be `long` or `short`. "
                                             "Only applicable to `FUTURES`/`SWAP`."),
        px: str = Field("", description="Order price. Only applicable to `limit`,`post_only`,`fok`,`ioc`,`mmp`,`mmp_and_post_only` order. "
                                        "When placing an option order, one of px/pxUsd/pxVol must be filled in, and only one can be filled in"),
        tgtCcy: str = Field("", description="Whether the target currency uses the quote or base currency. "
                                            "`base_ccy`: Base currency, `quote_ccy`: Quote currency. "
                                            "Only applicable to `SPOT` Market Orders. "
                                            "Default is `quote_ccy` for buy, `base_ccy` for sell"),
        reduceOnly: str | bool = Field("", description="Whether orders can only reduce in position size. "
                                                       "Valid options: `true` or `false`. The default value is `false`. "
                                                       "Only applicable to `MARGIN` orders, and `FUTURES`/`SWAP` orders in net mode. "
                                                       "Only applicable to `Futures mode` and `Multi-currency margin`."),
        stpMode: str = Field("", description="Self trade prevention mode: `cancel_maker`,`cancel_taker`,`cancel_both`. Cancel both does not support FOK. "
                                             "The account-level acctStpMode will be used to place orders by default. The default value of this field is `cancel_maker`. "
                                             "Users can log in to the webpage through the master account to modify this configuration. "
                                             "Users can also utilize the stpMode request parameter of the placing order endpoint to determine the stpMode of a certain order."),
        pxUsd: str = Field("", description="Place options orders in `USD`. Only applicable to options. "
                                           "When placing an option order, one of px/pxUsd/pxVol must be filled in, and only one can be filled in"),
        pxVol: str = Field("", description="Place options orders based on implied volatility, where 1 represents 100%. Only applicable to options. "
                                           "When placing an option order, one of px/pxUsd/pxVol must be filled in, and only one can be filled in"),
        banAmend: str | bool = Field("", description="Whether to disallow the system from amending the size of the SPOT Market Order. "
                                                     "Valid options: `true` or `false`. The default value is `false`. "
                                                     "If `true`, system will not amend and reject the market order if user does not have sufficient funds. "
                                                     "Only applicable to SPOT Market Orders"),
        attachAlgoOrds: list | None = Field(None, description="TP/SL information attached when placing order"),
    ):
        params = {
            'instId': instId, 'tdMode': tdMode, 'side': side, 'ordType': ordType, 'sz': sz, 'ccy': ccy,
            'clOrdId': clOrdId, 'tag': tag, 'posSide': posSide, 'px': px, 'reduceOnly': reduceOnly,
            'tgtCcy': tgtCcy, 'stpMode': stpMode, 'pxUsd': pxUsd, 'pxVol': pxVol, 'banAmend': banAmend,
        }
        if isinstance(reduceOnly, bool):
            params['reduceOnly'] = 'true' if reduceOnly else 'false'
        if isinstance(banAmend, bool):
            params['banAmend'] = 'true' if banAmend else 'false'
        if attachAlgoOrds:
            params['attachAlgoOrds'] = attachAlgoOrds
        return ACCOUNT.place_order(**params)

    @mcp.tool(
        title="Cancel an incomplete order",
        description="Cancel an incomplete order on OKX",
    )
    def cancel_order(
        instId: str = Field(description="Instrument ID, e.g. BTC-USDT"),
        ordId: str = Field("", description="Order ID. Either ordId or clOrdId is required. If both are passed, ordId will be used"),
        clOrdId: str = Field("", description="Client Order ID as assigned by the client"),
    ):
        return ACCOUNT.cancel_order(instId=instId, ordId=ordId, clOrdId=clOrdId)

    @mcp.tool(
        title="Get order details",
        description="Retrieve order details on OKX. "
                    "For a detailed schema of the output object, please read the resource at: `schema://trade/order`",
    )
    def get_trade_order(
        instId: str = Field(description="Instrument ID, e.g. BTC-USDT"),
        ordId: str = Field("", description="Order ID. Either ordId or clOrdId is required. If both are passed, ordId will be used"),
        clOrdId: str = Field("", description="Client Order ID as assigned by the client"),
    ):
        return ACCOUNT.get_order(instId=instId, ordId=ordId, clOrdId=clOrdId)

    @mcp.tool(
        title="Get incomplete order list",
        description="Retrieve all incomplete orders under the current OKX account. "
                    "For a detailed schema of the output object, please read the resource at: `schema://trade/order`",
    )
    def get_order_list(
        instType: str = Field("", description="Instrument type: `SPOT/MARGIN/SWAP/FUTURES/OPTION`"),
        instFamily: str = Field("", description="Instrument family. Applicable to `FUTURES/SWAP/OPTION`"),
        instId: str = Field("", description="Instrument ID, e.g. BTC-USD-200927"),
        state: str = Field("", description="State: `live`/`partially_filled`"),
        ordType: str = Field("", description="Order type. "
                                             "\n`market`: Market order"
                                             "\n`limit`: Limit order"
                                             "\n`post_only`: Post-only order"
                                             "\n`fok`: Fill-or-kill order"
                                             "\n`ioc`: Immediate-or-cancel order"
                                             "\n`optimal_limit_ioc`: Market order with immediate-or-cancel order"
                                             "\n`mmp`: Market Maker Protection (only applicable to Option in Portfolio Margin mode)"
                                             "\n`mmp_and_post_only`: Market Maker Protection and Post-only order(only applicable to Option in Portfolio Margin mode)"
                                             "\n`op_fok`: Simple options (fok)"),
    ):
        return ACCOUNT.get_order_list(
            instType=instType,
            instFamily=instFamily,
            instId=instId,
            state=state,
            ordType=ordType,
            limit=100,
        )

    @mcp.tool(
        title="Get Order History",
        description="Get completed orders on OKX which are placed in the last 7 days, including those placed 7 days ago but completed in the last 7 days."
                    "The incomplete orders that have been canceled are only reserved for 2 hours. "
                    "For a detailed schema of the output object, please read the resource at: `schema://trade/order`",
    )
    def get_orders_history(
        instType: str = Field(description="Instrument type: `SPOT/MARGIN/SWAP/FUTURES/OPTION`"),
        instFamily: str = Field("", description="Instrument family. Applicable to `FUTURES/SWAP/OPTION`"),
        instId: str = Field("", description="Instrument ID, e.g. BTC-USDT"),
        state: str = Field("", description="State: `canceled`/`filled`/`mmp_canceled`: Order canceled automatically due to Market Maker Protection"),
        category: str = Field("", description="Category: `twap/adl/full_liquidation/partial_liquidation/delivery`/`ddh`: Delta dynamic hedge"),
        ordType: str = Field("", description="Order type. "
                                             "\n`market`: Market order"
                                             "\n`limit`: Limit order"
                                             "\n`post_only`: Post-only order"
                                             "\n`fok`: Fill-or-kill order"
                                             "\n`ioc`: Immediate-or-cancel order"
                                             "\n`optimal_limit_ioc`: Market order with immediate-or-cancel order"
                                             "\n`mmp`: Market Maker Protection (only applicable to Option in Portfolio Margin mode)"
                                             "\n`mmp_and_post_only`: Market Maker Protection and Post-only order(only applicable to Option in Portfolio Margin mode)"
                                             "\n`op_fok`: Simple options (fok)"),
        limit: str | int = Field(20, description="Number of results per request. [1-100]. Default: 20"),
    ):
        return ACCOUNT.get_orders_history(
            instType=instType,
            instFamily=instFamily,
            instId=instId,
            state=state,
            ordType=ordType,
            limit=str(limit),
        )

    @mcp.tool(
        title="Close positions",
        description="Liquidate all positions in the designated trading product at market price on OKX",
    )
    def close_positions(
        instId: str = Field(description="Instrument ID, e.g. BTC-USDT"),
        mgnMode: str = Field(description="Margin mode: `cross`/`isolated`"),
        posSide: str = Field("", description="Position side. "
                                             "This parameter can be omitted in `net` mode, and the default value is `net`. You can only fill with `net`. "
                                             "This parameter must be filled in under the `long/short` mode. Fill in `long` for close-long and `short` for close-short."),
        ccy: str = Field("", description="Margin currency, required in the case of closing `cross` `MARGIN` position for `Futures mode`"),
        autoCxl: Any = Field("", description="Whether any pending orders for closing out needs to be automatically canceled when close position via a market order."
                                             "`false` or `true`, the default is `false`"),
        clOrdId: str = Field("", description="Client-supplied ID. A combination of case-sensitive alphanumerics, "
                                             "all numbers, or all letters of up to 32 characters."),
        tag: str = Field("", description="Order tag. A combination of case-sensitive alphanumerics, all numbers, or all letters of up to 16 characters"),
    ):
        return ACCOUNT.close_positions(
            instId=instId,
            mgnMode=mgnMode,
            posSide=posSide,
            ccy=ccy,
            autoCxl=autoCxl in [True, "true", "yes", 1],
            clOrdId=clOrdId,
            tag=tag,
        )


    @mcp.resource(
        uri="schema://trade/order",
        description="This resource is used to describe the schema of the trade order on OKX",
    )
    def schema_trade_order():
        return """
        instType	String	Instrument type `MARGIN/SWAP/FUTURES/OPTION`
        instId	String	Instrument ID
        tgtCcy	String	Order quantity unit setting for sz
            base_ccy: Base currency ,quote_ccy: Quote currency
            Only applicable to SPOT Market Orders
            Default is quote_ccy for buy, base_ccy for sell
        ccy	String	Margin currency
            Applicable to all isolated MARGIN orders and cross MARGIN orders in Futures mode, FUTURES and SWAP contracts.
        ordId	String	Order ID
        clOrdId	String	Client Order ID as assigned by the client
        tag	String	Order tag
        px	String	Price. For options, use coin as unit (e.g. BTC, ETH)
        pxUsd	String	Options price in USDOnly applicable to options; return "" for other instrument types
        pxVol	String	Implied volatility of the options orderOnly applicable to options; return "" for other instrument types
        pxType	String	Price type of options
            px: Place an order based on price, in the unit of coin (the unit for the request parameter px is BTC or ETH)
            pxVol: Place an order based on pxVol
            pxUsd: Place an order based on pxUsd, in the unit of USD (the unit for the request parameter px is USD)
        sz	String	Quantity to buy or sell
        pnl	String	Profit and loss (excluding the fee). Applicable to orders which have a trade and aim to close position. It always is 0 in other conditions
        ordType	String	Order type
            market: Market order
            limit: Limit order
            post_only: Post-only order
            fok: Fill-or-kill order
            ioc: Immediate-or-cancel order
            optimal_limit_ioc: Market order with immediate-or-cancel order
            mmp: Market Maker Protection (only applicable to Option in Portfolio Margin mode)
            mmp_and_post_only: Market Maker Protection and Post-only order(only applicable to Option in Portfolio Margin mode)
            op_fok: Simple options (fok)
        side	String	Order side
        posSide	String	Position side
        tdMode	String	Trade mode
        accFillSz	String	Accumulated fill quantity
            The unit is base_ccy for SPOT and MARGIN, e.g. BTC-USDT, the unit is BTC; For market orders, the unit both is base_ccy when the tgtCcy is base_ccy or quote_ccy;
            The unit is contract for FUTURES/SWAP/OPTION
        fillPx	String	Last filled price. If none is filled, it will return "".
        tradeId	String	Last traded ID
        fillSz	String	Last filled quantity
            The unit is base_ccy for SPOT and MARGIN, e.g. BTC-USDT, the unit is BTC; For market orders, the unit both is base_ccy when the tgtCcy is base_ccy or quote_ccy;
            The unit is contract for FUTURES/SWAP/OPTION
        fillTime	String	Last filled time
        avgPx	String	Average filled price. If none is filled, it will return "".
        state	String	State. canceled/live/partially_filled/filled/mmp_canceled
        lever	String	Leverage, from 0.01 to 125. Only applicable to MARGIN/FUTURES/SWAP
        stpMode	String	Self trade prevention mode
        attachAlgoClOrdId	String	Client-supplied Algo ID when placing order attaching TP/SL.
        tpTriggerPx	String	Take-profit trigger price.
        tpTriggerPxType	String	Take-profit trigger price type.
            last: last price
            index: index price
            mark: mark price
        tpOrdPx	String	Take-profit order price.
        slTriggerPx	String	Stop-loss trigger price.
        slTriggerPxType	String	Stop-loss trigger price type.
            last: last price
            index: index price
            mark: mark price
        slOrdPx	String	Stop-loss order price.
        attachAlgoOrds	Array of objects	TP/SL information attached when placing order
        > attachAlgoId	String	The order ID of attached TP/SL order. It can be used to identity the TP/SL order when amending. It will not be posted to algoId when placing TP/SL order after the general order is filled completely.
        > attachAlgoClOrdId	String	Client-supplied Algo ID when placing order attaching TP/SL
            A combination of case-sensitive alphanumerics, all numbers, or all letters of up to 32 characters.
            It will be posted to algoClOrdId when placing TP/SL order once the general order is filled completely.
        > tpOrdKind	String	TP order kind. condition/limit
        > tpTriggerPx	String	Take-profit trigger price.
        > tpTriggerPxType	String	Take-profit trigger price type.
            last: last price
            index: index price
            mark: mark price
        > tpOrdPx	String	Take-profit order price.
        > slTriggerPx	String	Stop-loss trigger price.
        > slTriggerPxType	String	Stop-loss trigger price type.
            last: last price
            index: index price
            mark: mark price
        > slOrdPx	String	Stop-loss order price.
        > sz	String	Size. Only applicable to TP order of split TPs
        > amendPxOnTriggerType	String	Whether to enable Cost-price SL. Only applicable to SL order of split TPs.
            0: disable, the default value
            1: Enable
        > amendPxOnTriggerType	String	Whether to enable Cost-price SL. Only applicable to SL order of split TPs.
            0: disable, the default value
            1: Enable
        > failCode	String	The error code when failing to place TP/SL order, e.g. 51020. The default is ""
        > failReason	String	The error reason when failing to place TP/SL order. The default is ""
        linkedAlgoOrd	Object	Linked SL order detail, only applicable to the order that is placed by one-cancels-the-other (OCO) order that contains the TP limit order.
        > algoId	String	Algo ID
        feeCcy	String	Fee currency
            For maker sell orders of Spot and Margin, this represents the quote currency. For all other cases, it represents the currency in which fees are charged.
        fee	String	Fee amount
            For Spot and Margin (excluding maker sell orders): accumulated fee charged by the platform, always negative
            For maker sell orders in Spot and Margin, Expiry Futures, Perpetual Futures and Options: accumulated fee and rebate (always in quote currency for maker sell orders in Spot and Margin)
        rebateCcy	String	Rebate currency
            For maker sell orders of Spot and Margin, this represents the base currency. For all other cases, it represents the currency in which rebates are paid.
        rebate	String	Rebate amount, only applicable to Spot and Margin
            For maker sell orders: Accumulated fee and rebate amount in base currency.
            For all other cases, it represents the maker rebate amount, always positive, return "" if no rebate.
        source	String	Order source
            6: The normal order triggered by the trigger order
            7: The normal order triggered by the TP/SL order
            13: The normal order triggered by the algo order
            25: The normal order triggered by the trailing stop order
            34: The normal order triggered by the chase order
        category	String	Category. normal/twap/adl/full_liquidation/partial_liquidation/delivery/ddh(Delta dynamic hedge)/auto_conversion
        reduceOnly	String	Whether the order can only reduce the position size. Valid options: true or false.
        isTpLimit	String	Whether it is TP limit order. true or false
        cancelSource	String	Code of the cancellation source.
        cancelSourceReason	String	Reason for the cancellation.
        quickMgnType	String	Quick Margin type, Only applicable to Quick Margin Mode of isolated margin. manual/auto_borrow/auto_repay
        algoClOrdId	String	Client-supplied Algo ID. There will be a value when algo order attaching algoClOrdId is triggered, or it will be "".
        algoId	String	Algo ID. There will be a value when algo order is triggered, or it will be "".
        tradeQuoteCcy	String	The quote currency used for trading.
        """



import os
import logging
import argparse
from fastmcp import FastMCP
from fastmcp.server.auth.providers.jwt import StaticTokenVerifier
from starlette.middleware.cors import CORSMiddleware

from . import (
    account,
    trading,
    config,
)

_LOGGER = logging.getLogger(__name__)


verifier = StaticTokenVerifier(tokens={
    config.MCP_AUTH_TOKEN: {
        "client_id": "mcp",
        "scopes": [],
    }
})
mcp = FastMCP(name="mcp-okx", version="0.1.0a2", auth=verifier)
account.add_tools(mcp)
trading.add_tools(mcp)


def main():
    mode = os.getenv("TRANSPORT")
    port = int(os.getenv("PORT", 0)) or 80
    parser = argparse.ArgumentParser(description="OKX MCP Server")
    parser.add_argument("--http", action="store_true", help="Use streamable HTTP mode instead of stdio")
    parser.add_argument("--host", default="0.0.0.0", help="Host to bind to (default: 0.0.0.0)")
    parser.add_argument("--port", type=int, default=port, help=f"Port to listen on (default: {port})")

    args = parser.parse_args()
    if args.http or mode == "http":
        app = mcp.http_app()
        app.add_middleware(
            CORSMiddleware,
            allow_origins=["*"],
            allow_credentials=True,
            allow_methods=["GET", "POST", "OPTIONS"],
            allow_headers=["*"],
            expose_headers=["mcp-session-id", "mcp-protocol-version"],
            max_age=86400,
        )
        mcp.run(transport="http", host=args.host, port=args.port)
    else:
        mcp.run()

if __name__ == "__main__":
    main()



from fastmcp import FastMCP
from pydantic import Field
from okx.Account import AccountAPI

from .config import *

ACCOUNT = AccountAPI(
    api_key=OKX_API_KEY,
    api_secret_key=OKX_API_SECRET,
    passphrase=OKX_PASSPHRASE,
    flag=OKX_TRADE_FLAG,
    domain=OKX_BASE_URL,
)


def add_tools(mcp: FastMCP):

    @mcp.tool(
        title="Get account configuration",
        description="Retrieve current OKX account configuration. "
                    "It is recommended to use this tool to obtain account configuration information before using all other tools",
    )
    def account_config():
        resp = ACCOUNT.get_account_config() or {}
        if resp.get("code"):
            return resp
        resp["_response_schema"] = """
        uid	String	Account ID of current request
        mainUid	String	Main Account ID of current request.
            The current request account is main account if uid = mainUid.
            The current request account is sub-account if uid != mainUid.
        acctLv	String	Account mode
            1: Spot mode
            2: Futures mode
            3: Multi-currency margin
            4: Portfolio margin
        acctStpMode	String	Account self-trade prevention mode: cancel_maker/cancel_taker/cancel_both
            The default value is cancel_maker. Users can log in to the webpage through the master account to modify this configuration
        posMode	String	Position mode. long_short_mode: long/short, only applicable to FUTURES/SWAP; net_mode: net
        autoLoan	Boolean	Whether to borrow coins automatically
            true: borrow coins automatically
            false: not borrow coins automatically
        feeType	String	Fee type
            0: fee is charged in the currency you receive from the trade
            1: fee is always charged in the quote currency of the trading pair
        level	String	The user level of the current real trading volume on the platform, e.g Lv1, which means regular user level
        levelTmp	String	Temporary experience user level of special users, e.g Lv1
        ctIsoMode	String	Contract isolated margin trading settings. automatic: Auto transfers; autonomy: Manual transfers
        mgnIsoMode	String	Margin isolated margin trading settings
            auto_transfers_ccy: New auto transfers, enabling both base and quote currency as the margin for isolated margin trading
            automatic: Auto transfers
            quick_margin: Quick Margin Mode (For new accounts, including subaccounts, some defaults will be automatic, and others will be quick_margin)
        roleType	String	Role type
            0: General user
            1: Leading trader
            2: Copy trader
        traderInsts	Array of strings	Leading trade instruments, only applicable to Leading trader
        spotRoleType	String	SPOT copy trading role type. 0: General user; 1: Leading trader; 2: Copy trader
        spotTraderInsts	Array of strings	Spot lead trading instruments, only applicable to lead trader
        opAuth	String	Whether the optional trading was activated. 0: not activate; 1: activated
        kycLv	String	Main account KYC level
            0: No verification
            1: level 1 completed
            2: level 2 completed
            3: level 3 completed
            If the request originates from a subaccount, kycLv is the KYC level of the main account.
            If the request originates from the main account, kycLv is the KYC level of the current account.
        label	String	API key note of current request API key. No more than 50 letters (case sensitive) or numbers, which can be pure letters or pure numbers
        ip	String	IP addresses that linked with current API key, separate with commas if more than one. It is an empty string "" if there is no IP bonded.
        perm	String	The permission of the current requesting API key or Access token. read_only: Read; trade: Trade; withdraw: Withdraw
        liquidationGear	String	The maintenance margin ratio level of liquidation alert
            3 and -1 means that you will get hourly liquidation alerts on app and channel "Position risk warning" when your margin level drops to or below 300%. -1 is the initial value which has the same effect as -3
            0 means that there is not alert
        enableSpotBorrow	Boolean	Whether borrow is allowed or not in Spot mode. true: Enabled; false: Disabled
        spotBorrowAutoRepay	Boolean	Whether auto-repay is allowed or not in Spot mode. true: Enabled; false: Disabled
        type	String	Account type
            0: Main account
            1: Standard sub-account
            2: Managed trading sub-account
            5: Custody trading sub-account - Copper
            9: Managed trading sub-account - Copper
            12: Custody trading sub-account - Komainu
        settleCcy	String	Current account's USD-margined contract settle currency
        settleCcyList	String	Current account's USD-margined contract settle currency list, like ["USD", "USDC", "USDG"]
        """
        return resp

    @mcp.tool(
        title="Get account balance",
        description="Retrieve a list of assets (with non-zero balance), remaining balance, and available amount in the OKX trading account",
    )
    def account_balance(
        ccy: str = Field("", description="Single currency or multiple currencies (no more than 20) separated with comma, e.g. BTC or BTC,ETH."),
    ):
        resp = ACCOUNT.get_account_balance(ccy) or {}
        if resp.get("code"):
            return resp
        resp["_response_schema"] = """
        totalEq: The total amount of equity in USD
        isoEq: Isolated margin equity in USD. Applicable to Futures mode/Multi-currency margin/Portfolio margin
        adjEq: Adjusted / Effective equity in USD.
            The net fiat value of the assets in the account that can provide margins for spot, expiry futures, perpetual futures and options under the cross-margin mode.
            In multi-ccy or PM mode, the asset and margin requirement will all be converted to USD value to process the order check or liquidation.
            Due to the volatility of each currency market, our platform calculates the actual USD value of each currency based on discount rates to balance market risks.
            Applicable to Spot mode/Multi-currency margin and Portfolio margin
        availEq: Account level available equity, excluding currencies that are restricted due to the collateralized borrowing limit.
            Applicable to Multi-currency margin/Portfolio margin
        ordFroz: Cross margin frozen for pending orders in USD. Only applicable to Spot mode/Multi-currency margin/Portfolio margin
        imr: Initial margin requirement in USD.
            The sum of initial margins of all open positions and pending orders under cross-margin mode in USD.
            Applicable to Spot mode/Multi-currency margin/Portfolio margin
        mmr: Maintenance margin requirement in USD.
            The sum of maintenance margins of all open positions and pending orders under cross-margin mode in USD.
            Applicable to Spot mode/Multi-currency margin/Portfolio margin
        borrowFroz: Potential borrowing IMR of the account in USD.
            Only applicable to Spot mode/Multi-currency margin/Portfolio margin. It is "" for other margin modes.
        mgnRatio: Maintenance margin ratio in USD. Applicable to Spot mode/Multi-currency margin/Portfolio margin
        notionalUsd: Notional value of positions in USD. Applicable to Spot mode/Multi-currency margin/Portfolio margin
        notionalUsdForBorrow: Notional value for Borrow in USD. Applicable to Spot mode/Multi-currency margin/Portfolio margin
        notionalUsdForSwap: Notional value of positions for Perpetual Futures in USD. Applicable to Multi-currency margin/Portfolio margin
        notionalUsdForFutures: Notional value of positions for Expiry Futures in USD. Applicable to Multi-currency margin/Portfolio margin
        notionalUsdForOption: Notional value of positions for Option in USD. Applicable to Spot mode/Multi-currency margin/Portfolio margin
        upl: Cross-margin info of unrealized profit and loss at the account level in USD. Applicable to Multi-currency margin/Portfolio margin
        details: Detailed asset information in all currencies
        details.ccy: Currency
        details.eq: Equity of currency
        details.cashBal: Cash balance
        details.disEq: Discount equity of currency in USD. Applicable to Spot mode(enabled spot borrow)/Multi-currency margin/Portfolio margin
        details.fixedBal: Frozen balance for Dip Sniper and Peak Sniper
        details.availBal: Available balance of currency
        details.frozenBal: Frozen balance of currency
        details.ordFrozen: Margin frozen for open orders. Applicable to Spot mode/Futures mode/Multi-currency margin
        details.liab: Liabilities of currency. It is a positive value, e.g. 21625.64; Applicable to Spot mode/Multi-currency margin/Portfolio margin
        details.uplLiab: Liabilities due to Unrealized loss of currency. Applicable to Multi-currency margin/Portfolio margin
        details.crossLiab: Cross liabilities of currency. Applicable to Spot mode/Multi-currency margin/Portfolio margin
        details.rewardBal: Trial fund balance
        details.isoLiab: Isolated liabilities of currency. Applicable to Multi-currency margin/Portfolio margin
        details.interest: Accrued interest of currency. It is a positive value, e.g. 9.01; Applicable to Spot mode/Multi-currency margin/Portfolio margin
        details.twap: Risk indicator of forced repayment. Divided into multiple levels from 0 to 5, the larger the number, 
            the more likely the forced repayment will be triggered. Applicable to Spot mode/Multi-currency margin/Portfolio margin
        details.frpType: Forced repayment (FRP) type. 0: no FRP; 1: user based FRP; 2: platform based FRP; 
            Return 1/2 when twap is >= 1, applicable to Spot mode/Multi-currency margin/Portfolio margin
        details.maxLoan: Max loan of currency. Applicable to cross of Spot mode/Multi-currency margin/Portfolio margin
        details.eqUsd: Equity in USD of currency
        details.borrowFroz: Potential borrowing IMR of currency in USD. Applicable to Multi-currency margin/Portfolio margin. It is "" for other margin modes.
        details.notionalLever: Leverage of currency. Applicable to Futures mode
        details.stgyEq: Strategy equity
        details.isoUpl: Isolated unrealized profit and loss of currency. Applicable to Futures mode/Multi-currency margin/Portfolio margin
        details.spotInUseAmt: Spot in use amount. Applicable to Portfolio margin
        details.clSpotInUseAmt: User-defined spot risk offset amount. Applicable to Portfolio margin
        details.maxSpotInUse: Max possible spot risk offset amount. Applicable to Portfolio margin
        details.spotIsoBal: Spot isolated balance. Applicable to copy trading. Applicable to Spot mode/Futures mode.
        details.smtSyncEq: Smart sync equity. The default is "0", only applicable to copy trader
        details.spotCopyTradingEq: Spot smart sync equity. The default is "0", only applicable to copy trader.
        details.spotBal: Spot balance. The unit is currency, e.g. BTC
        details.openAvgPx: Spot average cost price. The unit is USD
        details.accAvgPx: Spot accumulated cost price. The unit is USD
        details.spotUpl: Spot unrealized profit and loss. The unit is USD
        details.spotUplRatio: Spot unrealized profit and loss ratio
        details.totalPnl: Spot accumulated profit and loss. The unit is USD
        details.totalPnlRatio: Spot accumulated profit and loss ratio
        details.colRes: Platform level collateral restriction status.
            0: The restriction is not enabled.
            1: The restriction is not enabled. But the crypto is close to the platform's collateral limit.
            2: The restriction is enabled. This crypto can't be used as margin for your new orders. This may result in failed orders.
            But it will still be included in the account's adjusted equity and doesn't impact margin ratio.
        details.colBorrAutoConversion: Risk indicator of auto conversion. Divided into multiple levels from 1-5, the larger the number, the more likely the repayment will be triggered. The default will be 0, indicating there is no risk currently. 5 means this user is undergoing auto conversion now, 4 means this user will undergo auto conversion soon whereas 1/2/3 indicates there is a risk for auto conversion.
            Applicable to Spot mode/Futures mode/Multi-currency margin/Portfolio margin.
            When the total liability for each crypto set as collateral exceeds a certain percentage of the platform's total limit, the auto-conversion mechanism may be triggered.
            This may result in the automatic sale of excess collateral crypto if you've set this crypto as collateral and have large borrowings.
            To lower this risk, consider reducing your use of the crypto as collateral or reducing your liabilities.
        details.collateralEnabled: true: Collateral enabled; false: Collateral disabled. Applicable to Multi-currency margin
        details.autoLendStatus: Auto lend status
            unsupported: auto lend is not supported by this currency
            off: auto lend is supported but turned off
            pending: auto lend is turned on but pending matching
            active: auto lend is turned on and matched
        details.autoLendMtAmt: Auto lend currency matched amount.
            Return "0" when autoLendStatus is unsupported/off/pending. Return matched amount when autoLendStatus is active
        """
        return resp

    @mcp.tool(
        title="Get account positions",
        description="Retrieve information on your OKX positions. When the account is in net mode, net positions will be displayed, "
                    "and when the account is in long/short mode, long or short positions will be displayed. "
                    "Return in reverse chronological order using ctime.",
    )
    def account_positions(
        instType: str = Field("", description="Instrument type: "
                                              "`SPOT`: 币币现货/"
                                              "`MARGIN`: 币币杠杆/"
                                              "`SWAP`: 永续合约/"
                                              "`FUTURES`: 交割合约/"
                                              "`OPTION`: 期权."
                                              "`instId` will be checked against `instType` when both parameters are passed."),
        instId: str = Field("", description="Instrument ID, e.g. `BTC-USDT-SWAP`. Single instrument ID or multiple instrument IDs (no more than 10) separated with comma"),
        posId: str = Field("", description="Single position ID or multiple position IDs (no more than 20) separated with comma. "
                                           "There is attribute expiration, the posId and position information will be cleared if it is more than 30 days after the last full close position."),
    ):
        if str(instType).upper() in ["SPOT"]:
            instType = ""
        resp = ACCOUNT.get_positions(instType, instId=instId, posId=posId) or {}
        if resp.get("code"):
            return resp
        resp["_response_schema"] = """
        mgnMode: Margin mode
        posSide: Position side. long, pos is positive; short, pos is positive;
                 net (FUTURES/SWAP/OPTION: positive pos means long position and negative pos means short position. For MARGIN, 
                 pos is always positive, posCcy being base currency means long position, posCcy being quote currency means short position.)
        pos: Quantity of positions. In the isolated margin mode, when doing manual transfers, a position with pos of 0 will be generated after the deposit is transferred
        posCcy: Position currency, only applicable to MARGIN positions.
        availPos: Position that can be closed. Only applicable to MARGIN and OPTION. 
                  For MARGIN position, the rest of sz will be SPOT trading after the liability is repaid while closing the position. 
                  Please get the available reduce-only amount from `Get maximum available tradable amount` if you want to reduce the amount of SPOT trading as much as possible
        avgPx: Average open price. Under cross-margin mode, the entry price of expiry futures will update at settlement to the last settlement price, and when the position is opened or increased
        nonSettleAvgPx: Non-settlement entry price. The non-settlement entry price only reflects the average price at which the position is opened or increased. Applicable to cross FUTURES positions
        markPx: Latest Mark price
        upl: Unrealized profit and loss calculated by mark price
        uplRatio: Unrealized profit and loss ratio calculated by mark price
        uplLastPx: Unrealized profit and loss calculated by last price. Main usage is showing, actual value is upl
        uplRatioLastPx: Unrealized profit and loss ratio calculated by last price
        lever: Leverage. Not applicable to OPTION and positions of cross margin mode under Portfolio margin
        liqPx: Estimated liquidation price. Not applicable to OPTION
        imr: Initial margin requirement
        margin: Margin, can be added or reduced
        mgnRatio: Maintenance margin ratio
        mmr: Maintenance margin requirement
        liab: Liabilities, only applicable to MARGIN
        liabCcy: Liabilities currency
        optVal: Option Value, only applicable to OPTION
        pendingCloseOrdLiabVal: The amount of close orders of isolated margin liability
        notionalUsd: Notional value of positions in USD
        adl: Auto-deleveraging (ADL) indicator. Divided into 6 levels, from 0 to 5, the smaller the number, 
             the weaker the adl intensity. Only applicable to FUTURES/SWAP/OPTION
        ccy: Currency used for margin
        last: Latest traded price
        idxPx: Latest underlying index price
        usdPx: Latest USD price of the ccy on the market, only applicable to FUTURES/SWAP/OPTION
        bePx: Breakeven price
        deltaBS: delta: Black-Scholes Greeks in dollars, only applicable to OPTION
        deltaPA: delta: Greeks in coins, only applicable to OPTION
        gammaBS: gamma: Black-Scholes Greeks in dollars, only applicable to OPTION
        gammaPA: gamma: Greeks in coins, only applicable to OPTION
        thetaBS: theta：Black-Scholes Greeks in dollars, only applicable to OPTION
        thetaPA: theta：Greeks in coins, only applicable to OPTION
        vegaBS: vega：Black-Scholes Greeks in dollars, only applicable to OPTION
        vegaPA: vega：Greeks in coins, only applicable to OPTION
        spotInUseAmt: Spot in use amount. Applicable to Portfolio margin
        spotInUseCcy: Spot in use unit, e.g. BTC. Applicable to Portfolio margin
        clSpotInUseAmt: User-defined spot risk offset amount. Applicable to Portfolio margin
        maxSpotInUseAmt: Max possible spot risk offset amount. Applicable to Portfolio margin
        bizRefId: External business id, e.g. experience coupon id
        bizRefType: External business type
        realizedPnl: Realized profit and loss. Only applicable to FUTURES/SWAP/OPTION. 
                     realizedPnl=pnl+fee+fundingFee+liqPenalty+settledPnl
        settledPnl: Accumulated settled profit and loss (calculated by settlement price). Only applicable to cross FUTURES
        pnl: Accumulated pnl of closing order(s) (excluding the fee)
        fee: Accumulated fee. Negative number represents the user transaction fee charged by the platform.Positive number represents rebate.
        fundingFee: Accumulated funding fee
        liqPenalty: Accumulated liquidation penalty. It is negative when there is a value
        closeOrderAlgo: Close position algo orders attached to the position. 
                        This array will have values only after you request 'Place algo order' with closeFraction=1
        closeOrderAlgo.slTriggerPx: Stop-loss trigger price
        closeOrderAlgo.tpTriggerPx: Take-profit trigger price
        closeOrderAlgo.slTriggerPxType: Stop-loss trigger price type.
            last: last price
            index: index price
            mark: mark price
        closeOrderAlgo.tpTriggerPxType: Take-profit trigger price type.
            last: last price
            index: index price
            mark: mark price
        closeOrderAlgo.closeFraction: Fraction of position to be closed when the algo order is triggered
        """
        return resp

    @mcp.tool(
        title="Get account position risk",
        description="Obtain the overall holding risk of the OKX account",
    )
    def account_position_risk(
        instType: str = Field("", description="Instrument type: "
                                              "`MARGIN`: 币币杠杆/"
                                              "`SWAP`: 永续合约/"
                                              "`FUTURES`: 交割合约/"
                                              "`OPTION`: 期权."),
    ):
        resp = ACCOUNT.get_position_risk(instType) or {}
        if resp.get("code"):
            return resp
        resp["_response_schema"] = """
        adjEq: Adjusted / Effective equity in USD. Applicable to Multi-currency margin and Portfolio margin
        balData: Detailed asset information in all currencies
        balData.ccy: Currency
        balData.eq: Equity of currency
        balData.disEq: Discount equity of currency in USD
        posData: Detailed position information in all currencies
        posData.mgnMode: Margin mode: cross/isolated
        posData.instId: Instrument ID, e.g. BTC-USDT-SWAP
        posData.pos: Quantity of positions contract. In the isolated margin mode, when doing manual transfers,
            a position with pos of 0 will be generated after the deposit is transferred
        posData.posSide: Position side: long/short
            net (FUTURES/SWAP/OPTION: positive pos means long position and negative pos means short position. MARGIN: posCcy being base currency means long position, posCcy being quote currency means short position.)
        posData.posCcy: Position currency, only applicable to MARGIN positions
        posData.ccy: Currency used for margin
        posData.notionalCcy: Notional value of positions in coin
        posData.notionalUsd: Notional value of positions in USD
        """
        return resp



import os

def getenv(key: str, default: str = None, empty: bool = False):
    val = os.environ.get(key, default)
    if empty and not val:
        return default
    return val

OKX_BASE_URL = getenv("OKX_BASE_URL", "https://www.okx.com", True)
OKX_API_KEY = getenv("OKX_API_KEY", "-1")
OKX_API_SECRET = getenv("OKX_API_SECRET", "-1")
OKX_PASSPHRASE = getenv("OKX_PASSPHRASE", "-1")
OKX_TRADE_FLAG = getenv("OKX_TRADE_FLAG", "1")

MCP_AUTH_TOKEN = getenv("MCP_AUTH_TOKEN", OKX_API_KEY, True)
